from __future__ import with_statement # I am in fact lame enough to still be using python 2.5
import grfx
import math
import Box2D.Box2D as b2
from Box2D.Box2D import b2Vec2
import pyglet

gl = pyglet.gl

SCALE = 1.0/10.0

def has_tag(string, tag):
    if not string: return False
    for x in string.split(','):
        if x.strip() == tag:
            return True
    return False

def make_edge_from_svg_path(svgpath, b2world):
    """Takes an SVG path and creates a box2D edge chain for it"""
    r = []

    if not has_tag(svgpath.description, 'solid'): 
        return None

    for loop in svgpath.path:
        bd = b2.b2BodyDef()
        bd.position = (0,0)
        sd = b2.b2EdgeChainDef()
        data = [svgpath.transform((x[0], x[1])) for x in list(reversed(loop))]
        data = [(x[0]*SCALE, x[1]*SCALE) for x in data]

        sd.setVertices(data)
        sd.isALoop = False
        sd.density = 1.0
        sd.friction = 1.75
        sd.restitution = 0.1
        body = b2world.CreateBody(bd)
        shape = body.CreateShape(sd)
        r.append((body,shape))
    return r

def edges_from_svg(svg, b2world):
    """Extracts a list of edge chains from an SVG file"""
    return [y for y in  (make_edge_from_svg_path(x, b2world) for x in svg.paths) if y]


class CircleDef(object):
    def __init__(self, world, radius, density=1.0, friction=0.3, elasticity=0.1):
        self.world = world
        self._display_list = None
        
        sd = b2.b2CircleDef()
        sd.radius = radius
        sd.density = density
        sd.friction = friction
        sd.restitution = elasticity
        self.shape_def = sd
        
    def draw(self, instance):
        
        gl = pyglet.gl
        
        gl.glPushMatrix()
        gl.glTranslatef(instance.body.position.x, instance.body.position.y, 0)
        gl.glRotatef(instance.body.angle * (180/math.pi), 0, 0, 1)
        
        if not self._display_list:
            self._display_list = gl.glGenLists(1)
            gl.glNewList(self._display_list, gl.GL_COMPILE)
                        
            radius = instance.body.GetShapeList()[0].radius
            
            gl.glBegin(gl.GL_POLYGON)
            for i in xrange(0, 360, 20):
                r = i * (math.pi/180.0)
                gl.glVertex2f(math.cos(r)*radius, math.sin(r)*radius)
            gl.glEnd()
            
            #gl.glBegin(pyglet.gl.GL_LINE_STRIP)
            #gl.glVertex2f(0, 0)
            #gl.glVertex2f(0, radius)
            #gl.glEnd()
            
            gl.glEndList()
        
        gl.glCallList(self._display_list)
        gl.glPopMatrix()
        
class Circle(object):
    def __init__(self, definition, position=(0,0), angle=0):
        
        self.definition = definition
        bd=b2.b2BodyDef() 
        bd.position = position
        bd.angle = angle
        
        self.body = definition.world.CreateBody(bd) 
        self.body.CreateShape(definition.shape_def)
        self.body.SetMassFromShapes()
    
    def update(self, dt):
        pass
    
    def draw(self):
        self.definition.draw(self)

cached_defs = {}

def get_circle_definition(world, radius, density=1.0, friction=0.3):
    key = (radius,density,friction)
    if cached_defs.has_key(key):
        return cached_defs[key]
    cached_defs[key] = CircleDef(world, radius, density, friction)
    return cached_defs[key]