
class SceneManager(object):
    def __init__(self, scenes):
        self._scene = {}
        self.current_scene = scenes[0]
        
        for scene in scenes:
            self._scene[scene.name] = scene
            
            scene.switch_requested += self._switch_to_context

    def _switch_to_context(self, scene_name):
        print "in switch to context", scene_name
        self.current_scene = self._scene[scene_name]
        self.current_scene.on_activate()

class Signal(object):
    def __init__(self):
        """Initalize the signals dictionary"""
        self._handlers = []
            
    def emit(self, *args, **kwargs):
        for callback in self._handlers:
            callback(*args, **kwargs)
       
    def connect(self, func):
        self._handlers.append(func)
        return self

    __iadd__ = connect
    __call__ = emit

class Scene(object):
    def __init__(self, name):
        self.name = name
        
        self.switch_requested = Signal()
    
    def on_mouse_press(self, x, y, button, modifiers):
        print "Pressed", x, y, button
    
    def on_mouse_release(self, x, y, button, modifiers):
        print "Released", x, y, button
    
    def on_mouse_motion(self, x, y, dx, dy):
        pass
    
    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        pass
    
    def on_mouse_scroll(self, x, y, sx, sy):
        pass
    
    def on_key_press(self, symbol, modifiers):
        pass
    
    def on_activate(self):
        pass
    
    def update(self, dt):
        pass
    
    def predraw(self):
        pass
    
    def draw(self):
        pass
    
    def draw_ui(self):
        pass
    
    def switch_to(self, scene_name):
        self.switch_requested(scene_name)