import pyglet
import time
from grfx import screen

class Stat(object):
    def __init__(self, val, update_interval=0.5):
        self.last_updated = time.time()
        self.value = update_interval
        self.update_interval = 0.5

    def update(self, val, update_interval=None):
        if update_interval: self.update_interval = update_interval

        if time.time() - self.last_updated > self.update_interval:
            self.value = val
            self.last_updated = time.time()

    def val(self): return str(self.value)

class StatStack(object):

    def __init__(self):
        self.stats = {}
        self.label = None

    def update(self, statname, value, update_interval=0.5):
        if not self.stats.has_key(statname):
            self.stats[statname] = Stat(value, update_interval)
        else:
            self.stats[statname].update(value, update_interval)

    def draw(self):
        if not self.label:
            self.label = pyglet.text.Label("(nostats)",
                            font_name='Pic0',
                            font_size=12,
                            x=0, y=screen.height,
                            anchor_x='left', anchor_y='top',
                            color=(255, 255, 255,255))
        sorted_items = ((k,self.stats[k]) for k in sorted(self.stats.keys()))
        self.label.text = '\n'.join(n + ": " + v.val() for n,v in sorted_items)
        self.label.draw()

stats = StatStack()